@@ -30,18 +30,14 @@
 #include <config.h>
 
 #include "nautilus-navigation-action.h"
+
 #include "nautilus-navigation-window.h"
+#include "nautilus-navigation-window-slot.h"
 
-#include <gtk/gtkimage.h>
-#include <gtk/gtkimagemenuitem.h>
-#include <gtk/gtktoolbar.h>
-#include <gtk/gtkmenutoolbutton.h>
-
-static void nautilus_navigation_action_init       (NautilusNavigationAction *action);
-static void nautilus_navigation_action_class_init (NautilusNavigationActionClass *class);
-
-static GObjectClass *parent_class = NULL;
+#include <gtk/gtk.h>
+#include <eel/eel-gtk-extensions.h>
 
+G_DEFINE_TYPE (NautilusNavigationAction, nautilus_navigation_action, GTK_TYPE_ACTION);
 #define NAUTILUS_NAVIGATION_ACTION_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), NAUTILUS_TYPE_NAVIGATION_ACTION, NautilusNavigationActionPrivate))
 
 struct NautilusNavigationActionPrivate
@@ -59,30 +55,20 @@ enum
 	PROP_WINDOW
 };
 
-GType
-nautilus_navigation_action_get_type (void)
+static gboolean
+should_open_in_new_tab (void)
 {
-	static GType type = 0;
+	/* FIXME this is duplicated */
+	GdkEvent *event;
 
-	if (type == 0) {
-		const GTypeInfo type_info = {
-			sizeof (NautilusNavigationActionClass),
-			(GBaseInitFunc) NULL,
-			(GBaseFinalizeFunc) NULL,
-			(GClassInitFunc) nautilus_navigation_action_class_init,
-			(GClassFinalizeFunc) NULL,
-			NULL,
-			sizeof (NautilusNavigationAction),
-			0, /* n_preallocs */
-			(GInstanceInitFunc) nautilus_navigation_action_init,
-		};
-		
-		type = g_type_register_static (GTK_TYPE_ACTION,
-					       "NautilusNavigationAction",
-					       &type_info, 0);
+	event = gtk_get_current_event ();
+	if (event->type == GDK_BUTTON_PRESS || event->type == GDK_BUTTON_RELEASE) {
+		return event->button.button == 2;
 	}
 
-	return type;
+	gdk_event_free (event);
+
+	return FALSE;
 }
 
 static void
@@ -97,7 +83,7 @@ activate_back_or_forward_menu_item (GtkMenuItem *menu_item,
 
 	index = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (menu_item), "user_data"));
 
-	nautilus_navigation_window_back_or_forward (window, back, index);
+	nautilus_navigation_window_back_or_forward (window, back, index, should_open_in_new_tab ());
 }
 
 static void
@@ -117,13 +103,16 @@ fill_menu (NautilusNavigationWindow *window,
 	   GtkWidget *menu,
 	   gboolean back)
 {
+	NautilusNavigationWindowSlot *slot;
 	GtkWidget *menu_item;
 	int index;
 	GList *list;
 
 	g_assert (NAUTILUS_IS_NAVIGATION_WINDOW (window));
+
+	slot = NAUTILUS_NAVIGATION_WINDOW_SLOT (nautilus_window_get_active_slot (NAUTILUS_WINDOW (window)));
 	
-	list = back ? window->back_list : window->forward_list;
+	list = back ? slot->back_list : slot->forward_list;
 	index = 0;
 	while (list != NULL) {
 		menu_item = nautilus_bookmark_menu_item_new (NAUTILUS_BOOKMARK (list->data));
@@ -141,7 +130,6 @@ fill_menu (NautilusNavigationWindow *window,
 	}
 }
 
-
 static void
 show_menu_callback (GtkMenuToolButton *button,
 		    NautilusNavigationAction *action)
@@ -176,6 +164,30 @@ show_menu_callback (GtkMenuToolButton *button,
 	}
 }
 
+static gboolean
+proxy_button_press_event_cb (GtkButton *button,
+			     GdkEventButton *event,
+			     gpointer user_data)
+{
+	if (event->button == 2) {
+                g_signal_emit_by_name (button, "pressed", 0);
+	}
+
+	return FALSE;
+}
+
+static gboolean
+proxy_button_release_event_cb (GtkButton *button,
+			       GdkEventButton *event,
+			       gpointer user_data)
+{
+	if (event->button == 2) {
+                g_signal_emit_by_name (button, "released", 0);
+	}
+
+	return FALSE;
+}
+
 static void
 connect_proxy (GtkAction *action, GtkWidget *proxy)
 {
@@ -183,6 +195,7 @@ connect_proxy (GtkAction *action, GtkWidget *proxy)
 		NautilusNavigationAction *naction = NAUTILUS_NAVIGATION_ACTION (action);
 		GtkMenuToolButton *button = GTK_MENU_TOOL_BUTTON (proxy);
 		GtkWidget *menu;
+		GtkWidget *child;
 
 		/* set an empty menu, so the arrow button becomes sensitive */
 		menu = gtk_menu_new ();
@@ -193,19 +206,31 @@ connect_proxy (GtkAction *action, GtkWidget *proxy)
 		
 		g_signal_connect (proxy, "show-menu",
 				  G_CALLBACK (show_menu_callback), action);
+
+		/* Make sure that middle click works. Note that there is some code duplication
+		 * between here and nautilus-window-menus.c */
+		child = eel_gtk_menu_tool_button_get_button (button);
+		g_signal_connect (child, "button-press-event", G_CALLBACK (proxy_button_press_event_cb), NULL);
+		g_signal_connect (child, "button-release-event", G_CALLBACK (proxy_button_release_event_cb), NULL);
 	}
 
-	(* GTK_ACTION_CLASS (parent_class)->connect_proxy) (action, proxy);
+	(* GTK_ACTION_CLASS (nautilus_navigation_action_parent_class)->connect_proxy) (action, proxy);
 }
 
 static void
 disconnect_proxy (GtkAction *action, GtkWidget *proxy)
 {
 	if (GTK_IS_MENU_TOOL_BUTTON (proxy)) {
+		GtkWidget *child;
+		
 		g_signal_handlers_disconnect_by_func (proxy, G_CALLBACK (show_menu_callback), action);
+
+		child = eel_gtk_menu_tool_button_get_button (GTK_MENU_TOOL_BUTTON (proxy));
+		g_signal_handlers_disconnect_by_func (child, G_CALLBACK (proxy_button_press_event_cb), NULL);
+		g_signal_handlers_disconnect_by_func (child, G_CALLBACK (proxy_button_release_event_cb), NULL);
 	}
 
-	(* GTK_ACTION_CLASS (parent_class)->disconnect_proxy) (action, proxy);
+	(* GTK_ACTION_CLASS (nautilus_navigation_action_parent_class)->disconnect_proxy) (action, proxy);
 }
 
 static void
@@ -215,7 +240,7 @@ nautilus_navigation_action_finalize (GObject *object)
 
 	g_free (action->priv->arrow_tooltip);
 
-	(* G_OBJECT_CLASS (parent_class)->finalize) (object);
+	(* G_OBJECT_CLASS (nautilus_navigation_action_parent_class)->finalize) (object);
 }
 
 static void
@@ -277,8 +302,6 @@ nautilus_navigation_action_class_init (NautilusNavigationActionClass *class)
 	object_class->set_property = nautilus_navigation_action_set_property;
 	object_class->get_property = nautilus_navigation_action_get_property;
 
-	parent_class = g_type_class_peek_parent (class);
-
 	action_class->toolbar_item_type = GTK_TYPE_MENU_TOOL_BUTTON;
 	action_class->connect_proxy = connect_proxy;
 	action_class->disconnect_proxy = disconnect_proxy;
@@ -304,7 +327,7 @@ nautilus_navigation_action_class_init (NautilusNavigationActionClass *class)
                                          g_param_spec_object ("window",
                                                               "Window",
                                                               "The navigation window",
-                                                              G_TYPE_OBJECT,
+                                                              NAUTILUS_TYPE_NAVIGATION_WINDOW,
                                                               G_PARAM_READWRITE));
 
 	g_type_class_add_private (object_class, sizeof(NautilusNavigationActionPrivate));
