@@ -24,33 +24,36 @@
  
 #include <config.h>
 
-#include <eel/eel-debug.h>
-#include <eel/eel-gtk-extensions.h>
-#include <eel/eel-glib-extensions.h>
-#include <eel/eel-string.h>
-#include <eel/eel-stock-dialogs.h>
-#include <eel/eel-gdk-pixbuf-extensions.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
-#include <libnautilus-private/nautilus-debug-log.h>
+#include <gio/gio.h>
+
 #include <libnautilus-private/nautilus-dnd.h>
 #include <libnautilus-private/nautilus-bookmark.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
-#include <libnautilus-private/nautilus-sidebar-provider.h>
 #include <libnautilus-private/nautilus-module.h>
 #include <libnautilus-private/nautilus-file.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
 #include <libnautilus-private/nautilus-file-operations.h>
 #include <libnautilus-private/nautilus-trash-monitor.h>
 #include <libnautilus-private/nautilus-icon-names.h>
-#include <libnautilus-private/nautilus-window-info.h>
-#include <libnautilus-private/nautilus-window-slot-info.h>
-#include <gio/gio.h>
 
+#include <eel/eel-debug.h>
+#include <eel/eel-gtk-extensions.h>
+#include <eel/eel-glib-extensions.h>
+#include <eel/eel-graphic-effects.h>
+#include <eel/eel-string.h>
+#include <eel/eel-stock-dialogs.h>
+
+#include "nautilus-application.h"
 #include "nautilus-bookmark-list.h"
 #include "nautilus-places-sidebar.h"
 #include "nautilus-window.h"
+#include "nautilus-window-slot.h"
+
+#define DEBUG_FLAG NAUTILUS_DEBUG_PLACES
+#include <libnautilus-private/nautilus-debug.h>
 
 #define EJECT_BUTTON_XPAD 6
 #define ICON_CELL_XPAD 6
@@ -65,7 +68,7 @@ typedef struct {
 	char 	           *uri;
 	GtkListStore       *store;
 	GtkTreeModel       *filter_model;
-	NautilusWindowInfo *window;
+	NautilusWindow *window;
 	NautilusBookmarkList *bookmarks;
 	GVolumeMonitor *volume_monitor;
 
@@ -87,14 +90,13 @@ typedef struct {
 	GtkWidget *popup_menu_unmount_item;
 	GtkWidget *popup_menu_eject_item;
 	GtkWidget *popup_menu_rescan_item;
-	GtkWidget *popup_menu_format_item;
 	GtkWidget *popup_menu_empty_trash_item;
 	GtkWidget *popup_menu_start_item;
 	GtkWidget *popup_menu_stop_item;
 
 	/* volume mounting - delayed open process */
 	gboolean mounting;
-	NautilusWindowSlotInfo *go_to_after_mount_slot;
+	NautilusWindowSlot *go_to_after_mount_slot;
 	NautilusWindowOpenFlags go_to_after_mount_flags;
 
 	GtkTreePath *eject_highlight_path;
@@ -146,9 +148,6 @@ typedef enum {
 	SECTION_NETWORK,
 } SectionType;
 
-static void  nautilus_places_sidebar_iface_init        (NautilusSidebarIface         *iface);
-static void  sidebar_provider_iface_init               (NautilusSidebarProviderIface *iface);
-static GType nautilus_places_sidebar_provider_get_type (void);
 static void  open_selected_bookmark                    (NautilusPlacesSidebar        *sidebar,
 							GtkTreeModel                 *model,
 							GtkTreePath                  *path,
@@ -210,13 +209,7 @@ static GtkTreeModel *nautilus_shortcuts_model_filter_new (NautilusPlacesSidebar
 							  GtkTreeModel          *child_model,
 							  GtkTreePath           *root);
 
-G_DEFINE_TYPE_WITH_CODE (NautilusPlacesSidebar, nautilus_places_sidebar, GTK_TYPE_SCROLLED_WINDOW,
-			 G_IMPLEMENT_INTERFACE (NAUTILUS_TYPE_SIDEBAR,
-						nautilus_places_sidebar_iface_init));
-
-G_DEFINE_TYPE_WITH_CODE (NautilusPlacesSidebarProvider, nautilus_places_sidebar_provider, G_TYPE_OBJECT,
-			 G_IMPLEMENT_INTERFACE (NAUTILUS_TYPE_SIDEBAR_PROVIDER,
-						sidebar_provider_iface_init));
+G_DEFINE_TYPE (NautilusPlacesSidebar, nautilus_places_sidebar, GTK_TYPE_SCROLLED_WINDOW);
 
 static GdkPixbuf *
 get_eject_icon (gboolean highlighted)
@@ -232,7 +225,7 @@ get_eject_icon (gboolean highlighted)
 
 	if (highlighted) {
 		GdkPixbuf *high;
-		high = eel_gdk_pixbuf_render (eject, 1, 255, 255, 0, 0);
+		high = eel_create_spotlight_pixbuf (eject);
 		g_object_unref (eject);
 		eject = high;
 	}
@@ -248,6 +241,10 @@ is_built_in_bookmark (NautilusFile *file)
 	gboolean built_in;
 	gint idx;
 
+	if (nautilus_file_is_home (file)) {
+		return TRUE;
+	}
+
 	built_in = FALSE;
 
 	for (idx = 0; idx < G_USER_N_DIRECTORIES; idx++) {
@@ -434,10 +431,10 @@ update_places (NautilusPlacesSidebar *sidebar)
 	GVolume *volume;
 	int bookmark_count, index;
 	char *location, *mount_uri, *name, *desktop_path, *last_uri;
-	const gchar *path;
+	const gchar *path, *bookmark_name;
 	GIcon *icon;
 	GFile *root;
-	NautilusWindowSlotInfo *slot;
+	NautilusWindowSlot *slot;
 	char *tooltip;
 	GList *network_mounts;
 	NautilusFile *file;
@@ -457,8 +454,8 @@ update_places (NautilusPlacesSidebar *sidebar)
 	sidebar->devices_header_added = FALSE;
 	sidebar->bookmarks_header_added = FALSE;
 
-	slot = nautilus_window_info_get_active_slot (sidebar->window);
-	location = nautilus_window_slot_info_get_current_location (slot);
+	slot = nautilus_window_get_active_slot (sidebar->window);
+	location = nautilus_window_slot_get_current_uri (slot);
 
 	volume_monitor = sidebar->volume_monitor;
 
@@ -609,20 +606,20 @@ update_places (NautilusPlacesSidebar *sidebar)
 			continue;
 		}
 
-		name = nautilus_bookmark_get_name (bookmark);
+		bookmark_name = nautilus_bookmark_get_name (bookmark);
 		icon = nautilus_bookmark_get_icon (bookmark);
 		mount_uri = nautilus_bookmark_get_uri (bookmark);
 		tooltip = g_file_get_parse_name (root);
 
 		last_iter = add_place (sidebar, PLACES_BOOKMARK,
 				       SECTION_BOOKMARKS,
-				       name, icon, mount_uri,
+				       bookmark_name, icon, mount_uri,
 				       NULL, NULL, NULL, index,
 				       tooltip);
 		compare_for_selection (sidebar,
 				       location, mount_uri, last_uri,
 				       &last_iter, &select_path);
-		g_free (name);
+
 		g_object_unref (root);
 		g_object_unref (icon);
 		g_free (mount_uri);
@@ -637,18 +634,14 @@ update_places (NautilusPlacesSidebar *sidebar)
 
 	/* home folder */
 	if (strcmp (g_get_home_dir(), desktop_path) != 0) {
-		char *display_name;
-
 		mount_uri = nautilus_get_home_directory_uri ();
-		display_name = g_filename_display_basename (g_get_home_dir ());
 		icon = g_themed_icon_new (NAUTILUS_ICON_HOME);
 		last_iter = add_place (sidebar, PLACES_BUILT_IN,
 				       SECTION_COMPUTER,
-				       display_name, icon,
+				       _("Home"), icon,
 				       mount_uri, NULL, NULL, NULL, 0,
 				       _("Open your personal folder"));
 		g_object_unref (icon);
-		g_free (display_name);
 		compare_for_selection (sidebar,
 				       location, mount_uri, last_uri,
 				       &last_iter, &select_path);
@@ -1007,7 +1000,7 @@ desktop_location_changed_callback (gpointer user_data)
 }
 
 static void
-loading_uri_callback (NautilusWindowInfo *window,
+loading_uri_callback (NautilusWindow *window,
 		      char *location,
 		      NautilusPlacesSidebar *sidebar)
 {
@@ -1267,11 +1260,10 @@ bookmarks_drop_uris (NautilusPlacesSidebar *sidebar,
 {
 	NautilusBookmark *bookmark;
 	NautilusFile *file;
-	char *uri, *name;
+	char *uri;
 	char **uris;
 	int i;
 	GFile *location;
-	GIcon *icon;
 	
 	uris = gtk_selection_data_get_uris (selection_data);
 	if (!uris)
@@ -1290,9 +1282,7 @@ bookmarks_drop_uris (NautilusPlacesSidebar *sidebar,
 		location = g_file_new_for_uri (uri);
 		nautilus_file_unref (file);
 
-		name = nautilus_compute_title_for_location (location);
-		icon = g_themed_icon_new (NAUTILUS_ICON_FOLDER);
-		bookmark = nautilus_bookmark_new (location, name, TRUE, icon);
+		bookmark = nautilus_bookmark_new (location, NULL, NULL);
 
 		if (!nautilus_bookmark_list_contains (sidebar->bookmarks, bookmark)) {
 			nautilus_bookmark_list_insert_item (sidebar->bookmarks, bookmark, position++);
@@ -1300,8 +1290,6 @@ bookmarks_drop_uris (NautilusPlacesSidebar *sidebar,
 
 		g_object_unref (location);
 		g_object_unref (bookmark);
-		g_object_unref (icon);
-		g_free (name);
 		g_free (uri);
 	}
 
@@ -1555,7 +1543,6 @@ bookmarks_popup_menu_detach_cb (GtkWidget *attach_widget,
 	sidebar->popup_menu_unmount_item = NULL;
 	sidebar->popup_menu_eject_item = NULL;
 	sidebar->popup_menu_rescan_item = NULL;
-	sidebar->popup_menu_format_item = NULL;
 	sidebar->popup_menu_start_item = NULL;
 	sidebar->popup_menu_stop_item = NULL;
 	sidebar->popup_menu_empty_trash_item = NULL;
@@ -1592,12 +1579,10 @@ check_visibility (GMount           *mount,
 		  gboolean         *show_unmount,
 		  gboolean         *show_eject,
 		  gboolean         *show_rescan,
-		  gboolean         *show_format,
 		  gboolean         *show_start,
 		  gboolean         *show_stop)
 {
 	*show_mount = FALSE;
-	*show_format = FALSE;
 	*show_rescan = FALSE;
 	*show_start = FALSE;
 	*show_stop = FALSE;
@@ -1635,7 +1620,6 @@ bookmarks_check_popup_sensitivity (NautilusPlacesSidebar *sidebar)
 	gboolean show_unmount;
 	gboolean show_eject;
 	gboolean show_rescan;
-	gboolean show_format;
 	gboolean show_start;
 	gboolean show_stop;
 	gboolean show_empty_trash;
@@ -1664,7 +1648,7 @@ bookmarks_check_popup_sensitivity (NautilusPlacesSidebar *sidebar)
 	gtk_widget_set_sensitive (sidebar->popup_menu_empty_trash_item, !nautilus_trash_monitor_is_empty ());
 
  	check_visibility (mount, volume, drive,
- 			  &show_mount, &show_unmount, &show_eject, &show_rescan, &show_format, &show_start, &show_stop);
+ 			  &show_mount, &show_unmount, &show_eject, &show_rescan, &show_start, &show_stop);
 
 	/* We actually want both eject and unmount since eject will unmount all volumes. 
 	 * TODO: hide unmount if the drive only has a single mountable volume 
@@ -1674,12 +1658,11 @@ bookmarks_check_popup_sensitivity (NautilusPlacesSidebar *sidebar)
 			   (!strcmp (uri, "trash:///"));
 
 	gtk_widget_set_visible (sidebar->popup_menu_separator_item,
-		      show_mount || show_unmount || show_eject || show_format || show_empty_trash);
+		      show_mount || show_unmount || show_eject || show_empty_trash);
 	gtk_widget_set_visible (sidebar->popup_menu_mount_item, show_mount);
 	gtk_widget_set_visible (sidebar->popup_menu_unmount_item, show_unmount);
 	gtk_widget_set_visible (sidebar->popup_menu_eject_item, show_eject);
 	gtk_widget_set_visible (sidebar->popup_menu_rescan_item, show_rescan);
-	gtk_widget_set_visible (sidebar->popup_menu_format_item, show_format);
 	gtk_widget_set_visible (sidebar->popup_menu_start_item, show_start);
 	gtk_widget_set_visible (sidebar->popup_menu_stop_item, show_stop);
 	gtk_widget_set_visible (sidebar->popup_menu_empty_trash_item, show_empty_trash);
@@ -1745,17 +1728,21 @@ volume_mounted_cb (GVolume *volume,
 
 		if (sidebar->go_to_after_mount_slot != NULL) {
 			if ((sidebar->go_to_after_mount_flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW) == 0) {
-				nautilus_window_slot_info_open_location (sidebar->go_to_after_mount_slot, location,
-									 NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE,
-									 sidebar->go_to_after_mount_flags, NULL);
+				nautilus_window_slot_open_location (sidebar->go_to_after_mount_slot, location,
+								    NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE,
+								    sidebar->go_to_after_mount_flags, NULL);
 			} else {
-				NautilusWindow *cur, *new;
-				
+				NautilusApplication *app;
+				NautilusWindow *new, *cur;
+
 				cur = NAUTILUS_WINDOW (sidebar->window);
-				new = nautilus_application_create_navigation_window (cur->application,
+				app = nautilus_application_dup_singleton ();
+				new = nautilus_application_create_navigation_window (app,
 										     NULL,
 										     gtk_window_get_screen (GTK_WINDOW (cur)));
 				nautilus_window_go_to (new, location);
+
+				g_object_unref (app);
 			}
 		}
 
@@ -1797,7 +1784,7 @@ open_selected_bookmark (NautilusPlacesSidebar *sidebar,
 			GtkTreePath	      *path,
 			NautilusWindowOpenFlags	      flags)
 {
-	NautilusWindowSlotInfo *slot;
+	NautilusWindowSlot *slot;
 	GtkTreeIter iter;
 	GFile *location;
 	char *uri;
@@ -1813,24 +1800,27 @@ open_selected_bookmark (NautilusPlacesSidebar *sidebar,
 	gtk_tree_model_get (model, &iter, PLACES_SIDEBAR_COLUMN_URI, &uri, -1);
 
 	if (uri != NULL) {
-		nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
-				    "activate from places sidebar window=%p: %s",
-				    sidebar->window, uri);
+		DEBUG ("Activating bookmark %s", uri);
+
 		location = g_file_new_for_uri (uri);
 		/* Navigate to the clicked location */
 		if ((flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW) == 0) {
-			slot = nautilus_window_info_get_active_slot (sidebar->window);
-			nautilus_window_slot_info_open_location (slot, location,
-								 NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE,
-								 flags, NULL);
+			slot = nautilus_window_get_active_slot (sidebar->window);
+			nautilus_window_slot_open_location (slot, location,
+							    NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE,
+							    flags, NULL);
 		} else {
 			NautilusWindow *cur, *new;
+			NautilusApplication *app;
 			
 			cur = NAUTILUS_WINDOW (sidebar->window);
-			new = nautilus_application_create_navigation_window (cur->application,
+			app = nautilus_application_dup_singleton ();
+			new = nautilus_application_create_navigation_window (app,
 									     NULL,
 									     gtk_window_get_screen (GTK_WINDOW (cur)));
 			nautilus_window_go_to (new, location);
+
+			g_object_unref (app);
 		}
 		g_object_unref (location);
 		g_free (uri);
@@ -1850,7 +1840,7 @@ open_selected_bookmark (NautilusPlacesSidebar *sidebar,
 
 			g_assert (sidebar->go_to_after_mount_slot == NULL);
 
-			slot = nautilus_window_info_get_active_slot (sidebar->window);
+			slot = nautilus_window_get_active_slot (sidebar->window);
 			sidebar->go_to_after_mount_slot = slot;
 			eel_add_weak_pointer (&(sidebar->go_to_after_mount_slot));
 
@@ -2002,7 +1992,7 @@ unmount_done (gpointer data)
 	NautilusWindow *window;
 
 	window = data;
-	nautilus_window_info_set_initiated_unmount (window, FALSE);
+	nautilus_window_set_initiated_unmount (window, FALSE);
 	g_object_unref (window);
 }
 
@@ -2011,7 +2001,7 @@ do_unmount (GMount *mount,
 	    NautilusPlacesSidebar *sidebar)
 {
 	if (mount != NULL) {
-		nautilus_window_info_set_initiated_unmount (sidebar->window, TRUE);
+		nautilus_window_set_initiated_unmount (sidebar->window, TRUE);
 		nautilus_file_operations_unmount_mount_full (NULL, mount, FALSE, TRUE,
 							     unmount_done,
 							     g_object_ref (sidebar->window));
@@ -2056,7 +2046,7 @@ drive_eject_cb (GObject *source_object,
 	char *name;
 
 	window = user_data;
-	nautilus_window_info_set_initiated_unmount (window, FALSE);
+	nautilus_window_set_initiated_unmount (window, FALSE);
 	g_object_unref (window);
 
 	error = NULL;
@@ -2085,7 +2075,7 @@ volume_eject_cb (GObject *source_object,
 	char *name;
 
 	window = user_data;
-	nautilus_window_info_set_initiated_unmount (window, FALSE);
+	nautilus_window_set_initiated_unmount (window, FALSE);
 	g_object_unref (window);
 
 	error = NULL;
@@ -2114,7 +2104,7 @@ mount_eject_cb (GObject *source_object,
 	char *name;
 
 	window = user_data;
-	nautilus_window_info_set_initiated_unmount (window, FALSE);
+	nautilus_window_set_initiated_unmount (window, FALSE);
 	g_object_unref (window);
 
 	error = NULL;
@@ -2142,15 +2132,15 @@ do_eject (GMount *mount,
 
 	mount_op = gtk_mount_operation_new (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (sidebar))));
 	if (mount != NULL) {
-		nautilus_window_info_set_initiated_unmount (sidebar->window, TRUE);
+		nautilus_window_set_initiated_unmount (sidebar->window, TRUE);
 		g_mount_eject_with_operation (mount, 0, mount_op, NULL, mount_eject_cb,
 					      g_object_ref (sidebar->window));
 	} else if (volume != NULL) {
-		nautilus_window_info_set_initiated_unmount (sidebar->window, TRUE);
+		nautilus_window_set_initiated_unmount (sidebar->window, TRUE);
 		g_volume_eject_with_operation (volume, 0, mount_op, NULL, volume_eject_cb,
 					      g_object_ref (sidebar->window));
 	} else if (drive != NULL) {
-		nautilus_window_info_set_initiated_unmount (sidebar->window, TRUE);
+		nautilus_window_set_initiated_unmount (sidebar->window, TRUE);
 		g_drive_eject_with_operation (drive, 0, mount_op, NULL, drive_eject_cb,
 					      g_object_ref (sidebar->window));
 	}
@@ -2296,13 +2286,6 @@ rescan_shortcut_cb (GtkMenuItem           *item,
 	g_object_unref (drive);
 }
 
-static void
-format_shortcut_cb (GtkMenuItem           *item,
-		    NautilusPlacesSidebar *sidebar)
-{
-	g_spawn_command_line_async ("gfloppy", NULL);
-}
-
 static void
 drive_start_cb (GObject      *source_object,
 		GAsyncResult *res,
@@ -2365,7 +2348,7 @@ drive_stop_cb (GObject *source_object,
 	char *name;
 
 	window = user_data;
-	nautilus_window_info_set_initiated_unmount (window, FALSE);
+	nautilus_window_set_initiated_unmount (window, FALSE);
 	g_object_unref (window);
 
 	error = NULL;
@@ -2402,7 +2385,7 @@ stop_shortcut_cb (GtkMenuItem           *item,
 		GMountOperation *mount_op;
 
 		mount_op = gtk_mount_operation_new (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (sidebar))));
-		nautilus_window_info_set_initiated_unmount (sidebar->window, TRUE);
+		nautilus_window_set_initiated_unmount (sidebar->window, TRUE);
 		g_drive_stop (drive, G_MOUNT_UNMOUNT_NONE, mount_op, NULL, drive_stop_cb,
 			      g_object_ref (sidebar->window));
 		g_object_unref (mount_op);
@@ -2427,6 +2410,30 @@ bookmarks_key_press_event_cb (GtkWidget             *widget,
 
   modifiers = gtk_accelerator_get_default_mod_mask ();
 
+  if ((event->keyval == GDK_KEY_Return ||
+       event->keyval == GDK_KEY_KP_Enter ||
+       event->keyval == GDK_KEY_ISO_Enter ||
+       event->keyval == GDK_KEY_space)) {
+
+      GtkTreeModel *model;
+      GtkTreePath *path;
+      NautilusWindowOpenFlags flags = 0;
+
+      if ((event->state & modifiers) == GDK_SHIFT_MASK) {
+          flags = NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB;
+      } else if ((event->state & modifiers) == GDK_CONTROL_MASK) {
+          flags = NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW;
+      }
+
+      model = gtk_tree_view_get_model (sidebar->tree_view);
+      gtk_tree_view_get_cursor (sidebar->tree_view, &path, NULL);
+
+      open_selected_bookmark (sidebar, model, path, flags);
+
+      gtk_tree_path_free (path);
+      return TRUE;
+  }
+
   if (event->keyval == GDK_KEY_Down &&
       (event->state & modifiers) == GDK_MOD1_MASK) {
       return eject_or_unmount_selection (sidebar);
@@ -2535,13 +2542,6 @@ bookmarks_build_popup_menu (NautilusPlacesSidebar *sidebar)
 	gtk_widget_show (item);
 	gtk_menu_shell_append (GTK_MENU_SHELL (sidebar->popup_menu), item);
 
-	item = gtk_menu_item_new_with_mnemonic (_("_Format"));
-	sidebar->popup_menu_format_item = item;
-	g_signal_connect (item, "activate",
-		    G_CALLBACK (format_shortcut_cb), sidebar);
-	gtk_widget_show (item);
-	gtk_menu_shell_append (GTK_MENU_SHELL (sidebar->popup_menu), item);
-
 	item = gtk_menu_item_new_with_mnemonic (_("_Start"));
 	sidebar->popup_menu_start_item = item;
 	g_signal_connect (item, "activate",
@@ -2717,9 +2717,7 @@ bookmarks_motion_event_cb (GtkWidget             *widget,
 			   NautilusPlacesSidebar *sidebar)
 {
 	GtkTreePath *path;
-	GtkTreeModel *model;
 
-	model = GTK_TREE_MODEL (sidebar->filter_model);
 	path = NULL;
 
 	if (over_eject_button (sidebar, event->x, event->y, &path)) {
@@ -2800,7 +2798,7 @@ bookmarks_edited (GtkCellRenderer       *cell,
 	bookmark = nautilus_bookmark_list_item_at (sidebar->bookmarks, index);
 
 	if (bookmark != NULL) {
-		nautilus_bookmark_set_name (bookmark, new_text);
+		nautilus_bookmark_set_custom_name (bookmark, new_text);
 	}
 }
 
@@ -2855,11 +2853,8 @@ icon_cell_renderer_func (GtkTreeViewColumn *column,
 			 GtkTreeIter *iter,
 			 gpointer user_data)
 {
-	NautilusPlacesSidebar *sidebar;
 	PlaceType type;
 
-	sidebar = user_data;
-
 	gtk_tree_model_get (model, iter,
 			    PLACES_SIDEBAR_COLUMN_ROW_TYPE, &type,
 			    -1);
@@ -2934,6 +2929,7 @@ nautilus_places_sidebar_init (NautilusPlacesSidebar *sidebar)
 	GtkTreeViewColumn *col;
 	GtkCellRenderer   *cell;
 	GtkTreeSelection  *selection;
+	GtkStyleContext   *style;
 
 	sidebar->volume_monitor = g_volume_monitor_get ();
 	
@@ -2948,6 +2944,9 @@ nautilus_places_sidebar_init (NautilusPlacesSidebar *sidebar)
 	tree_view = GTK_TREE_VIEW (gtk_tree_view_new ());
 	gtk_tree_view_set_headers_visible (tree_view, FALSE);
 
+	style = gtk_widget_get_style_context (GTK_WIDGET (tree_view));
+	gtk_style_context_add_class (style, "NautilusSidebar");
+
 	col = GTK_TREE_VIEW_COLUMN (gtk_tree_view_column_new ());
 
 	/* initial padding */
@@ -3147,20 +3146,10 @@ nautilus_places_sidebar_dispose (GObject *object)
 		sidebar->eject_highlight_path = NULL;
 	}
 
-	if (sidebar->store != NULL) {
-		g_object_unref (sidebar->store);
-		sidebar->store = NULL;
-	}
-
-	if (sidebar->volume_monitor != NULL) {
-		g_object_unref (sidebar->volume_monitor);
-		sidebar->volume_monitor = NULL;
-	}
-
-	if (sidebar->bookmarks != NULL) {
-		g_object_unref (sidebar->bookmarks);
-		sidebar->bookmarks = NULL;
-	}
+	g_clear_object (&sidebar->store);
+	g_clear_object (&sidebar->volume_monitor);
+	g_clear_object (&sidebar->bookmarks);
+	g_clear_object (&sidebar->filter_model);
 
 	eel_remove_weak_pointer (&(sidebar->go_to_after_mount_slot));
 
@@ -3179,61 +3168,20 @@ nautilus_places_sidebar_class_init (NautilusPlacesSidebarClass *class)
 	GTK_WIDGET_CLASS (class)->style_set = nautilus_places_sidebar_style_set;
 }
 
-static const char *
-nautilus_places_sidebar_get_sidebar_id (NautilusSidebar *sidebar)
-{
-	return NAUTILUS_PLACES_SIDEBAR_ID;
-}
-
-static char *
-nautilus_places_sidebar_get_tab_label (NautilusSidebar *sidebar)
-{
-	return g_strdup (_("Places"));
-}
-
-static char *
-nautilus_places_sidebar_get_tab_tooltip (NautilusSidebar *sidebar)
-{
-	return g_strdup (_("Show Places"));
-}
-
-static GdkPixbuf *
-nautilus_places_sidebar_get_tab_icon (NautilusSidebar *sidebar)
-{
-	return NULL;
-}
-
-static void
-nautilus_places_sidebar_is_visible_changed (NautilusSidebar *sidebar,
-					     gboolean         is_visible)
-{
-	/* Do nothing */
-}
-
-static void
-nautilus_places_sidebar_iface_init (NautilusSidebarIface *iface)
-{
-	iface->get_sidebar_id = nautilus_places_sidebar_get_sidebar_id;
-	iface->get_tab_label = nautilus_places_sidebar_get_tab_label;
-	iface->get_tab_tooltip = nautilus_places_sidebar_get_tab_tooltip;
-	iface->get_tab_icon = nautilus_places_sidebar_get_tab_icon;
-	iface->is_visible_changed = nautilus_places_sidebar_is_visible_changed;
-}
-
 static void
 nautilus_places_sidebar_set_parent_window (NautilusPlacesSidebar *sidebar,
-					   NautilusWindowInfo *window)
+					   NautilusWindow *window)
 {
-	NautilusWindowSlotInfo *slot;
+	NautilusWindowSlot *slot;
 
 	sidebar->window = window;
 
-	slot = nautilus_window_info_get_active_slot (window);
+	slot = nautilus_window_get_active_slot (window);
 
 	sidebar->bookmarks = nautilus_bookmark_list_new ();
-	sidebar->uri = nautilus_window_slot_info_get_current_location (slot);
+	sidebar->uri = nautilus_window_slot_get_current_uri (slot);
 
-	g_signal_connect_object (sidebar->bookmarks, "contents_changed",
+	g_signal_connect_object (sidebar->bookmarks, "changed",
 				 G_CALLBACK (update_places),
 				 sidebar, G_CONNECT_SWAPPED);
 
@@ -3274,9 +3222,8 @@ nautilus_places_sidebar_style_set (GtkWidget *widget,
 	update_places (sidebar);
 }
 
-static NautilusSidebar *
-nautilus_places_sidebar_create (NautilusSidebarProvider *provider,
-				NautilusWindowInfo *window)
+GtkWidget *
+nautilus_places_sidebar_new (NautilusWindow *window)
 {
 	NautilusPlacesSidebar *sidebar;
 	
@@ -3284,30 +3231,9 @@ nautilus_places_sidebar_create (NautilusSidebarProvider *provider,
 	nautilus_places_sidebar_set_parent_window (sidebar, window);
 	g_object_ref_sink (sidebar);
 
-	return NAUTILUS_SIDEBAR (sidebar);
+	return GTK_WIDGET (sidebar);
 }
 
-static void 
-sidebar_provider_iface_init (NautilusSidebarProviderIface *iface)
-{
-	iface->create = nautilus_places_sidebar_create;
-}
-
-static void
-nautilus_places_sidebar_provider_init (NautilusPlacesSidebarProvider *sidebar)
-{
-}
-
-static void
-nautilus_places_sidebar_provider_class_init (NautilusPlacesSidebarProviderClass *class)
-{
-}
-
-void
-nautilus_places_sidebar_register (void)
-{
-        nautilus_module_add_type (nautilus_places_sidebar_provider_get_type ());
-}
 
 /* Drag and drop interfaces */
 
