@@ -23,32 +23,16 @@
 
 #include <config.h>
 #include "nautilus-query-editor.h"
+#include "nautilus-src-marshal.h"
+#include "nautilus-window-slot.h"
 
 #include <string.h>
-#include <libnautilus-private/nautilus-marshal.h>
 #include <glib/gi18n.h>
+#include <gio/gio.h>
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-glib-extensions.h>
-#include <eel/eel-mime-extensions.h>
 #include <gdk/gdkkeysyms.h>
-#include <gtk/gtkbindings.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtkframe.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtkcombobox.h>
-#include <gtk/gtkdialog.h>
-#include "gtk/gtkliststore.h"
-#include "gtk/gtktreeselection.h"
-#include "gtk/gtkscrolledwindow.h"
-#include <gtk/gtkfilechooserbutton.h>
-#include "gtk/gtkcelllayout.h"
-#include "gtk/gtktooltips.h"
-#include "gtk/gtkcellrenderertext.h"
-#include <libgnomevfs/gnome-vfs-utils.h>
-#include <libgnomevfs/gnome-vfs-mime-info.h>
+#include <gtk/gtk.h>
 
 typedef enum {
 	NAUTILUS_QUERY_EDITOR_ROW_LOCATION,
@@ -87,11 +71,12 @@ struct NautilusQueryEditorDetails {
 	gboolean is_visible;
 	GtkWidget *invisible_vbox;
 	GtkWidget *visible_vbox;
-	GtkTooltips *tooltips;
 
 	GList *rows;
+	char *last_set_query_text;
 	
 	NautilusSearchBar *bar;
+	NautilusWindowSlot *slot;
 };
 
 enum {
@@ -165,7 +150,11 @@ nautilus_query_editor_dispose (GObject *object)
 	NautilusQueryEditor *editor;
 
 	editor = NAUTILUS_QUERY_EDITOR (object);
-	
+
+	if (editor->details->typing_timeout_id) {
+		g_source_remove (editor->details->typing_timeout_id);
+		editor->details->typing_timeout_id = 0;
+	}
 
 	if (editor->details->bar != NULL) {
 		g_signal_handlers_disconnect_by_func (editor->details->entry,
@@ -179,11 +168,6 @@ nautilus_query_editor_dispose (GObject *object)
 		eel_remove_weak_pointer (&editor->details->bar);
 	}
 
-	if (editor->details->tooltips) {
-		g_object_unref (editor->details->tooltips);
-		editor->details->tooltips = NULL;
-	}
-	
 	EEL_CALL_PARENT (G_OBJECT_CLASS, dispose, (object));
 }
 
@@ -203,20 +187,20 @@ nautilus_query_editor_class_init (NautilusQueryEditorClass *class)
 		              G_SIGNAL_RUN_LAST,
 		              G_STRUCT_OFFSET (NautilusQueryEditorClass, changed),
 		              NULL, NULL,
-		              nautilus_marshal_VOID__OBJECT_BOOLEAN,
+		              nautilus_src_marshal_VOID__OBJECT_BOOLEAN,
 		              G_TYPE_NONE, 2, NAUTILUS_TYPE_QUERY, G_TYPE_BOOLEAN);
 
 	signals[CANCEL] =
 		g_signal_new ("cancel",
 		              G_TYPE_FROM_CLASS (class),
-		              G_SIGNAL_RUN_LAST | GTK_RUN_ACTION,
+		              G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
 		              G_STRUCT_OFFSET (NautilusQueryEditorClass, cancel),
 		              NULL, NULL,
 		              g_cclosure_marshal_VOID__VOID,
 		              G_TYPE_NONE, 0);
 
 	binding_set = gtk_binding_set_by_class (class);
-	gtk_binding_entry_add_signal (binding_set, GDK_Escape, 0, "cancel", 0);
+	gtk_binding_entry_add_signal (binding_set, GDK_KEY_Escape, 0, "cancel", 0);
 }
 
 static void
@@ -276,8 +260,8 @@ static GtkWidget *
 location_row_create_widgets (NautilusQueryEditorRow *row)
 {
 	GtkWidget *chooser;
-	
-	chooser = gtk_file_chooser_button_new (_("Select folder search in"),
+
+	chooser = gtk_file_chooser_button_new (_("Select folder to search in"),
 					       GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
 	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (chooser), TRUE);
 	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser),
@@ -300,9 +284,15 @@ location_row_add_to_query (NautilusQueryEditorRow *row,
 	char *folder, *uri;
 	
 	folder = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (row->type_widget));
-	uri = gnome_vfs_get_uri_from_local_path (folder);
+	if (folder == NULL) {
+		/* I don't know why, but i got NULL here on initial search in browser mode
+		   even with the location set to the homedir in create_widgets... */
+		folder = g_strdup (g_get_home_dir ());
+	}
+	
+	uri = g_filename_to_uri (folder, NULL, NULL);
 	g_free (folder);
-
+		
 	nautilus_query_set_location (query, uri);
 	g_free (uri);
 }
@@ -324,7 +314,7 @@ location_add_rows_from_query (NautilusQueryEditor    *editor,
 	if (uri == NULL) {
 		return;
 	}
-	folder = gnome_vfs_get_local_path_from_uri (uri);
+	folder = g_filename_from_uri (uri, NULL, NULL);
 	g_free (uri);
 	if (folder == NULL) {
 		return;
@@ -381,6 +371,7 @@ struct {
 	},
 	{ N_("Music"),
 	  { "application/ogg",
+	    "audio/x-vorbis+ogg",
 	    "audio/ac3",
 	    "audio/basic",
 	    "audio/midi",
@@ -528,20 +519,27 @@ type_combo_changed (GtkComboBox *combo_box, NautilusQueryEditorRow *row)
 		GtkWidget *toplevel;
 		GtkTreeSelection *selection;
 
-		mime_infos = eel_mime_get_availible_mime_types ();
+		mime_infos = g_content_types_get_registered ();
 
 		store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
 		for (l = mime_infos; l != NULL; l = l->next) {
 			GtkTreeIter iter;
-			EelMimeTypeInfo *info = l->data;
+			char *mime_type = l->data;
+			char *description;
 
+			description = g_content_type_get_description (mime_type);
+			if (description == NULL) {
+				description = g_strdup (mime_type);
+			}
+			
 			gtk_list_store_append (store, &iter);
 			gtk_list_store_set (store, &iter,
-					    0, info->description,
-					    1, info->mime_type,
+					    0, description,
+					    1, mime_type,
 					    -1);
 			
-			eel_mime_type_info_free (info);
+			g_free (mime_type);
+			g_free (description);
 		}
 		g_list_free (mime_infos);
 		
@@ -550,7 +548,7 @@ type_combo_changed (GtkComboBox *combo_box, NautilusQueryEditorRow *row)
 		toplevel = gtk_widget_get_toplevel (GTK_WIDGET (combo_box));
 		dialog = gtk_dialog_new_with_buttons (_("Select type"),
 						      GTK_WINDOW (toplevel),
-						      GTK_DIALOG_NO_SEPARATOR,
+						      0,
 						      GTK_STOCK_OK, GTK_RESPONSE_OK,
 						      NULL);
 		gtk_window_set_default_size (GTK_WINDOW (dialog), 400, 600);
@@ -563,7 +561,7 @@ type_combo_changed (GtkComboBox *combo_box, NautilusQueryEditorRow *row)
 						     GTK_SHADOW_IN);
 		
 		gtk_widget_show (scrolled);
-		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), scrolled, TRUE, TRUE, 6);
+		gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), scrolled, TRUE, TRUE, 6);
 
 		treeview = gtk_tree_view_new ();
 		gtk_tree_view_set_model (GTK_TREE_VIEW (treeview),
@@ -731,12 +729,9 @@ remove_group_types_from_list (char **group_types, GList *mime_types)
 	GList *l, *next;
 	char **group_type;
 	char *mime_type;
-	gboolean found;
 
 	group_type = group_types;
 	while (*group_type != NULL) {
-		found = FALSE;
-
 		for (l = mime_types; l != NULL; l = next) {
 			mime_type = l->data;
 			next = l->next;
@@ -793,7 +788,7 @@ type_add_rows_from_query (NautilusQueryEditor    *editor,
 	for (l = mime_types; l != NULL; l = l->next) {
 		mime_type = l->data;
 
-		desc = gnome_vfs_mime_get_value (mime_type, "description");
+		desc = g_content_type_get_description (mime_type);
 		if (desc == NULL) {
 			desc = mime_type;
 		}
@@ -806,9 +801,8 @@ type_add_rows_from_query (NautilusQueryEditor    *editor,
 		gtk_combo_box_set_active_iter  (GTK_COMBO_BOX (row->type_widget),
 						&iter);
 	}
-	
-	eel_g_list_free_deep (mime_types);
 
+	g_list_free_full (mime_types, g_free);
 }
 
 /* End of row types */
@@ -904,10 +898,10 @@ nautilus_query_editor_add_row (NautilusQueryEditor *editor,
 	gtk_widget_show (hbox);
 	gtk_box_pack_start (GTK_BOX (editor->details->visible_vbox), hbox, FALSE, FALSE, 0);
 
-	combo = gtk_combo_box_new_text ();
+	combo = gtk_combo_box_text_new ();
 	row->combo = combo;
 	for (i = 0; i < NAUTILUS_QUERY_EDITOR_ROW_LAST; i++) {
-		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), gettext (row_type[i].name));
+		gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), gettext (row_type[i].name));
 	}
 	gtk_widget_show (combo);
 	gtk_box_pack_start (GTK_BOX (hbox), combo, FALSE, FALSE, 0);
@@ -931,9 +925,8 @@ nautilus_query_editor_add_row (NautilusQueryEditor *editor,
 
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (remove_row_cb), row);
-
-	gtk_tooltips_set_tip (editor->details->tooltips, button,
-			      _("Remove this criterium from the search"), NULL);
+	gtk_widget_set_tooltip_text (button,
+				     _("Remove this criterion from the search"));
 	
 	gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
 
@@ -957,6 +950,7 @@ static void
 nautilus_query_editor_init (NautilusQueryEditor *editor)
 {
 	GtkWidget *hbox, *label, *button;
+	char *label_markup;
 
 	editor->details = g_new0 (NautilusQueryEditorDetails, 1);
 	editor->details->is_visible = TRUE;
@@ -977,7 +971,9 @@ nautilus_query_editor_init (NautilusQueryEditor *editor)
 	gtk_widget_show (hbox);
 	
 	label = gtk_label_new ("");
-	gtk_label_set_markup (GTK_LABEL (label), _("<b>Search Folder</b>"));
+	label_markup = g_strconcat ("<b>", _("Search Folder"), "</b>", NULL);
+	gtk_label_set_markup (GTK_LABEL (label), label_markup);
+	g_free (label_markup);
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 	gtk_widget_show (label);
 	
@@ -988,12 +984,8 @@ nautilus_query_editor_init (NautilusQueryEditor *editor)
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (edit_clicked), editor);
 
-	editor->details->tooltips = gtk_tooltips_new ();
-	g_object_ref (editor->details->tooltips);
-	gtk_object_sink (GTK_OBJECT (editor->details->tooltips));
-
-	gtk_tooltips_set_tip (editor->details->tooltips, button,
-			      _("Edit the saved search"), NULL);
+	gtk_widget_set_tooltip_text (button,
+				     _("Edit the saved search"));
 }
 
 void
@@ -1023,8 +1015,8 @@ finish_first_line (NautilusQueryEditor *editor, GtkWidget *hbox, gboolean use_go
 	
 	gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
 
-	gtk_tooltips_set_tip (editor->details->tooltips, button,
-			      _("Add a new criterium to this search"), NULL);
+	gtk_widget_set_tooltip_text (button,
+				     _("Add a new criterion to this search"));
 
 	if (!editor->details->is_indexed) {
 		if (use_go) {
@@ -1033,9 +1025,9 @@ finish_first_line (NautilusQueryEditor *editor, GtkWidget *hbox, gboolean use_go
 			button = gtk_button_new_with_label (_("Reload"));
 		}
 		gtk_widget_show (button);
-		
-		gtk_tooltips_set_tip (editor->details->tooltips, button,
-				      _("Perform or update the search"), NULL);
+
+		gtk_widget_set_tooltip_text (button,
+					     _("Perform or update the search"));
 		
 		g_signal_connect (button, "clicked",
 				  G_CALLBACK (go_search_cb), editor);
@@ -1048,6 +1040,7 @@ static void
 setup_internal_entry (NautilusQueryEditor *editor)
 {
 	GtkWidget *hbox, *label;
+	char *label_markup;
 	
 	/* Create visible part: */
 	hbox = gtk_hbox_new (FALSE, 6);
@@ -1055,7 +1048,9 @@ setup_internal_entry (NautilusQueryEditor *editor)
 	gtk_box_pack_start (GTK_BOX (editor->details->visible_vbox), hbox, FALSE, FALSE, 0);
 
 	label = gtk_label_new ("");
-	gtk_label_set_markup_with_mnemonic (GTK_LABEL (label), _("<b>_Search for:</b>"));
+	label_markup = g_strconcat ("<b>", _("_Search for:"), "</b>", NULL);
+	gtk_label_set_markup_with_mnemonic (GTK_LABEL (label), label_markup);
+	g_free (label_markup);
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 
@@ -1159,6 +1154,10 @@ nautilus_query_editor_get_query (NautilusQueryEditor *editor)
 	GList *l;
 	NautilusQueryEditorRow *row;
 
+	if (editor == NULL || editor->details == NULL || editor->details->entry == NULL) {
+		return NULL;
+	}
+
 	query_text = gtk_entry_get_text (GTK_ENTRY (editor->details->entry));
 
 	/* Empty string is a NULL query */
@@ -1183,11 +1182,16 @@ nautilus_query_editor_clear_query (NautilusQueryEditor *editor)
 {
 	editor->details->change_frozen = TRUE;
 	gtk_entry_set_text (GTK_ENTRY (editor->details->entry), "");
+
+	g_free (editor->details->last_set_query_text);
+	editor->details->last_set_query_text = g_strdup ("");
+
 	editor->details->change_frozen = FALSE;
 }
 
 GtkWidget *
-nautilus_query_editor_new (gboolean start_hidden, gboolean is_indexed)
+nautilus_query_editor_new (gboolean start_hidden,
+			   gboolean is_indexed)
 {
 	GtkWidget *editor;
 
@@ -1203,10 +1207,45 @@ nautilus_query_editor_new (gboolean start_hidden, gboolean is_indexed)
 	return editor;
 }
 
+static void
+detach_from_external_entry (NautilusQueryEditor *editor)
+{
+	if (editor->details->bar != NULL) {
+		nautilus_search_bar_return_entry (editor->details->bar);
+		g_signal_handlers_block_by_func (editor->details->entry,
+						 entry_activate_cb,
+						 editor);
+		g_signal_handlers_block_by_func (editor->details->entry,
+						 entry_changed_cb,
+						 editor);
+	}
+}
+
+static void
+attach_to_external_entry (NautilusQueryEditor *editor)
+{
+	if (editor->details->bar != NULL) {
+		nautilus_search_bar_borrow_entry (editor->details->bar);
+		g_signal_handlers_unblock_by_func (editor->details->entry,
+						   entry_activate_cb,
+						   editor);
+		g_signal_handlers_unblock_by_func (editor->details->entry,
+						   entry_changed_cb,
+						   editor);
+
+		editor->details->change_frozen = TRUE;
+		gtk_entry_set_text (GTK_ENTRY (editor->details->entry),
+				    editor->details->last_set_query_text);
+		editor->details->change_frozen = FALSE;
+	}
+}
+
 GtkWidget*
 nautilus_query_editor_new_with_bar (gboolean start_hidden,
 				    gboolean is_indexed,
-				    NautilusSearchBar *bar)
+				    gboolean start_attached,
+				    NautilusSearchBar *bar,
+				    NautilusWindowSlot *slot)
 {
 	GtkWidget *entry;
 	NautilusQueryEditor *editor;
@@ -1218,9 +1257,21 @@ nautilus_query_editor_new_with_bar (gboolean start_hidden,
 
 	editor->details->bar = bar;
 	eel_add_weak_pointer (&editor->details->bar);
-	
+
+	editor->details->slot = slot;
+
 	entry = nautilus_search_bar_borrow_entry (bar);
 	setup_external_entry (editor, entry);
+	if (!start_attached) {
+		detach_from_external_entry (editor);
+	}
+
+	g_signal_connect_object (slot, "active",
+				 G_CALLBACK (attach_to_external_entry),
+				 editor, G_CONNECT_SWAPPED);
+	g_signal_connect_object (slot, "inactive",
+				 G_CALLBACK (detach_from_external_entry),
+				 editor, G_CONNECT_SWAPPED);
 	
 	return GTK_WIDGET (editor);
 }
@@ -1229,7 +1280,7 @@ void
 nautilus_query_editor_set_query (NautilusQueryEditor *editor, NautilusQuery *query)
 {
 	NautilusQueryEditorRowType type;
-	const char *text;
+	char *text;
 
 	if (!query) {
 		nautilus_query_editor_clear_query (editor);
@@ -1237,8 +1288,9 @@ nautilus_query_editor_set_query (NautilusQueryEditor *editor, NautilusQuery *que
 	}
 
 	text = nautilus_query_get_text (query);
+
 	if (!text) {
-		text = "";
+		text = g_strdup ("");
 	}
 
 	editor->details->change_frozen = TRUE;
@@ -1249,4 +1301,7 @@ nautilus_query_editor_set_query (NautilusQueryEditor *editor, NautilusQuery *que
 	}
 	
 	editor->details->change_frozen = FALSE;
+
+	g_free (editor->details->last_set_query_text);
+	editor->details->last_set_query_text = text;
 }
