@@ -37,6 +37,7 @@
 #include "nautilus-window-slot.h"
 #include "nautilus-navigation-window-slot.h"
 #include "nautilus-trash-bar.h"
+#include "nautilus-view-factory.h"
 #include "nautilus-x-content-bar.h"
 #include "nautilus-zoom-control.h"
 #include "nautilus-navigation-window-pane.h"
@@ -52,19 +53,17 @@
 #include <gdk/gdkx.h>
 #include <glib/gi18n.h>
 #include <libnautilus-extension/nautilus-location-widget-provider.h>
-#include <libnautilus-private/nautilus-debug-log.h>
 #include <libnautilus-private/nautilus-file-attributes.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
 #include <libnautilus-private/nautilus-file.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-metadata.h>
-#include <libnautilus-private/nautilus-mime-actions.h>
 #include <libnautilus-private/nautilus-module.h>
 #include <libnautilus-private/nautilus-monitor.h>
 #include <libnautilus-private/nautilus-search-directory.h>
-#include <libnautilus-private/nautilus-view-factory.h>
-#include <libnautilus-private/nautilus-window-info.h>
-#include <libnautilus-private/nautilus-window-slot-info.h>
+
+#define DEBUG_FLAG NAUTILUS_DEBUG_WINDOW
+#include <libnautilus-private/nautilus-debug.h>
 
 /* FIXME bugzilla.gnome.org 41243: 
  * We should use inheritance instead of these special cases
@@ -106,28 +105,14 @@ static void load_new_location                         (NautilusWindowSlot
 static void location_has_really_changed               (NautilusWindowSlot         *slot);
 static void update_for_new_location                   (NautilusWindowSlot         *slot);
 
-void
-nautilus_window_report_selection_changed (NautilusWindowInfo *window)
-{
-	if (window->details->temporarily_ignore_view_signals) {
-		return;
-	}
-
-	g_signal_emit_by_name (window, "selection_changed");
-}
-
 /* set_displayed_location:
  */
 static void
 set_displayed_location (NautilusWindowSlot *slot, GFile *location)
 {
-	NautilusWindow *window;
         GFile *bookmark_location;
         gboolean recreate;
-	char *name;
 
-	window = slot->pane->window;
-        
         if (slot->current_location_bookmark == NULL || location == NULL) {
                 recreate = TRUE;
         } else {
@@ -142,10 +127,8 @@ set_displayed_location (NautilusWindowSlot *slot, GFile *location)
 			g_object_unref (slot->last_location_bookmark);
                 }
 		slot->last_location_bookmark = slot->current_location_bookmark;
-		name = g_file_get_basename (location);
 		slot->current_location_bookmark = (location == NULL) ? NULL
-                        : nautilus_bookmark_new (location, name, FALSE, NULL);
-		g_free (name);
+                        : nautilus_bookmark_new (location, NULL, NULL);
         }
 }
 
@@ -263,9 +246,9 @@ handle_go_forward (NautilusNavigationWindowSlot *navigation_slot,
 }
 
 static void
-handle_go_elsewhere (NautilusWindowSlot *slot, GFile *location)
+handle_go_elsewhere (NautilusWindowSlot *slot,
+		     GFile *location)
 {
-#if !NEW_UI_COMPLETE
 	NautilusNavigationWindowSlot *navigation_slot;
 
 	if (NAUTILUS_IS_NAVIGATION_WINDOW_SLOT (slot)) {
@@ -289,7 +272,6 @@ handle_go_elsewhere (NautilusWindowSlot *slot, GFile *location)
 			}
 		}
 	}
-#endif
 }
 
 void
@@ -423,7 +405,7 @@ update_history (NautilusWindowSlot *slot,
         case NAUTILUS_LOCATION_CHANGE_STANDARD:
         case NAUTILUS_LOCATION_CHANGE_FALLBACK:
                 nautilus_window_slot_add_current_location_to_history_list (slot);
-                handle_go_elsewhere (slot, new_location);
+		handle_go_elsewhere (slot, new_location);
                 return;
         case NAUTILUS_LOCATION_CHANGE_RELOAD:
                 /* for reload there is no work to do */
@@ -491,25 +473,23 @@ nautilus_window_slot_open_location_full (NautilusWindowSlot *slot,
 	char *old_uri, *new_uri;
 	int new_slot_position;
 	GList *l;
-	gboolean target_spatial, target_navigation, target_same;
+	gboolean target_navigation, target_same;
 	gboolean is_desktop;
+	NautilusApplication *app;
 
 	window = slot->pane->window;
 
         target_window = NULL;
 	target_slot = NULL;
-	target_spatial = target_navigation = target_same = FALSE;
 
 	old_uri = nautilus_window_slot_get_location_uri (slot);
 	if (old_uri == NULL) {
 		old_uri = g_strdup ("(none)");
 	}
 	new_uri = g_file_get_uri (location);
-	nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
-			    "window %p open location: old=\"%s\", new=\"%s\"",
-			    window,
-			    old_uri,
-			    new_uri);
+
+	DEBUG ("Opening location, old: %s, new: %s", old_uri, new_uri);
+
 	g_free (old_uri);
 	g_free (new_uri);
 
@@ -533,9 +513,6 @@ nautilus_window_slot_open_location_full (NautilusWindowSlot *slot,
 			} else {
 				target_same = TRUE;
 			}
-		} else if (NAUTILUS_IS_SPATIAL_WINDOW (window)) {
-			/* don't always use browser: if source is spatial, target is spatial */
-			target_spatial = TRUE;
 		} else if (flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW) {
 			/* if it's specified to open a new window, and we're not using spatial,
 			 * the target is a navigation.
@@ -543,9 +520,6 @@ nautilus_window_slot_open_location_full (NautilusWindowSlot *slot,
 			target_navigation = TRUE;
 		}
                 break;
-        case NAUTILUS_WINDOW_OPEN_IN_SPATIAL :
-		target_spatial = TRUE;
-		break;
         case NAUTILUS_WINDOW_OPEN_IN_NAVIGATION :
 		target_navigation = TRUE;
                 break;
@@ -555,17 +529,19 @@ nautilus_window_slot_open_location_full (NautilusWindowSlot *slot,
                 return;
         }
 
+	app = nautilus_application_dup_singleton ();
+
 	/* now get/create the window according to the mode */
 	if (target_same) {
 		target_window = window;
 	} else if (target_navigation) {
 		target_window = nautilus_application_create_navigation_window
-			(window->application,
+			(app,
 			 NULL,
 			 gtk_window_get_screen (GTK_WINDOW (window)));
 	} else {
 		target_window = nautilus_application_get_spatial_window
-			(window->application,
+			(app,
 			 window,
 			 NULL,
 			 location,
@@ -573,6 +549,8 @@ nautilus_window_slot_open_location_full (NautilusWindowSlot *slot,
 			 &existing);
 	}
 
+	g_object_unref (app);
+
 	/* if the spatial window is already showing, present it and set the
 	 * new selection, if present.
 	 */
@@ -661,74 +639,6 @@ nautilus_window_slot_open_location_full (NautilusWindowSlot *slot,
 	}
 }
 
-void
-nautilus_window_slot_open_location (NautilusWindowSlot *slot,
-				    GFile *location,
-				    gboolean close_behind)
-{
-	NautilusWindowOpenFlags flags;
-
-	flags = 0;
-	if (close_behind) {
-		flags = NAUTILUS_WINDOW_OPEN_FLAG_CLOSE_BEHIND;
-	}
-	
-	nautilus_window_slot_open_location_full (slot, location,
-						 NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE,
-						 flags, NULL, NULL, NULL);
-}
-
-void
-nautilus_window_slot_open_location_with_selection (NautilusWindowSlot *slot,
-						   GFile *location,
-						   GList *selection,
-						   gboolean close_behind)
-{
-	NautilusWindowOpenFlags flags;
-
-	flags = 0;
-	if (close_behind) {
-		flags = NAUTILUS_WINDOW_OPEN_FLAG_CLOSE_BEHIND;
-	}
-	nautilus_window_slot_open_location_full (slot, location,
-						 NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE,
-						 flags, selection, NULL, NULL);
-}
-
-
-void
-nautilus_window_slot_go_home (NautilusWindowSlot *slot, gboolean new_tab)
-{			      
-	GFile *home;
-	NautilusWindowOpenFlags flags;
-
-	g_return_if_fail (NAUTILUS_IS_WINDOW_SLOT (slot));
-
-	if (new_tab) {
-		flags = NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB;
-	} else {
-		flags = 0;
-	}
-
-	home = g_file_new_for_path (g_get_home_dir ());
-	nautilus_window_slot_open_location_full (slot, home, 
-						 NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE, 
-						 flags, NULL, NULL, NULL);
-	g_object_unref (home);
-}
-
-#if 0
-static char *
-nautilus_window_slot_get_view_label (NautilusWindowSlot *slot)
-{
-	const NautilusViewInfo *info;
-
-	info = nautilus_view_factory_lookup (nautilus_window_slot_get_content_view_id (slot));
-
-	return g_strdup (info->label);
-}
-#endif
-
 static char *
 nautilus_window_slot_get_view_error_label (NautilusWindowSlot *slot)
 {
@@ -821,7 +731,7 @@ report_callback (NautilusWindowSlot *slot,
 /*
  * begin_location_change
  * 
- * Change a window's location.
+ * Change a window slot's location.
  * @window: The NautilusWindow whose location should be changed.
  * @location: A url specifying the location to load
  * @new_selection: The initial selection to present after loading the location
@@ -845,7 +755,6 @@ begin_location_change (NautilusWindowSlot *slot,
 		       NautilusWindowGoToCallback callback,
 		       gpointer user_data)
 {
-	NautilusWindow *window;
         NautilusDirectory *directory;
         NautilusFile *file;
 	gboolean force_reload;
@@ -857,10 +766,6 @@ begin_location_change (NautilusWindowSlot *slot,
                   || type == NAUTILUS_LOCATION_CHANGE_FORWARD
                   || distance == 0);
 
-	window = slot->pane->window;
-        g_assert (NAUTILUS_IS_WINDOW (window));
-        g_object_ref (window);
-
 	end_location_change (slot);
 
 	nautilus_window_slot_set_allow_stop (slot, TRUE);
@@ -925,8 +830,6 @@ begin_location_change (NautilusWindowSlot *slot,
 				       NAUTILUS_FILE_ATTRIBUTE_MOUNT,
                                        got_file_info_for_view_selection_callback,
 				       slot);
-
-        g_object_unref (window);
 }
 
 static void
@@ -1009,7 +912,7 @@ setup_new_spatial_window (NautilusWindowSlot *slot, NautilusFile *file)
 		} else {
 			/* If there is a pending selection, we want to scroll to an item in
 			 * the pending selection list. */
-			scroll_string = g_file_get_uri (slot->pending_selection->data);
+			scroll_string = nautilus_file_get_uri (slot->pending_selection->data);
 		}
 
 		/* scroll_string might be NULL if there was no saved scroll position. */
@@ -1030,14 +933,12 @@ mount_not_mounted_callback (GObject *source_object,
 			    gpointer user_data)
 {
 	MountNotMountedData *data;
-	NautilusWindow *window;
 	NautilusWindowSlot *slot;
 	GError *error;
 	GCancellable *cancellable;
 
 	data = user_data;
 	slot = data->slot;
-	window = slot->pane->window;
 	cancellable = data->cancellable;
 	g_free (data);
 
@@ -1281,14 +1182,13 @@ create_content_view (NautilusWindowSlot *slot,
         	g_object_ref (view);
         } else {
                 /* create a new content view */
-		view = nautilus_view_factory_create (view_id,
-						     NAUTILUS_WINDOW_SLOT_INFO (slot));
+		view = nautilus_view_factory_create (view_id, slot);
 
                 eel_accessibility_set_name (view, _("Content View"));
                 eel_accessibility_set_description (view, _("View of the current folder"));
 
                 slot->new_content_view = view;
-		nautilus_window_slot_connect_content_view (slot, slot->new_content_view);
+		nautilus_window_connect_content_view (window, slot->new_content_view);
         }
 
 	/* Actually load the pending location and selection: */
@@ -1328,7 +1228,6 @@ load_new_location (NautilusWindowSlot *slot,
 	NautilusWindow *window;
 	GList *selection_copy;
 	NautilusView *view;
-	char *uri;
 
 	g_assert (slot != NULL);
 	g_assert (location != NULL);
@@ -1343,18 +1242,14 @@ load_new_location (NautilusWindowSlot *slot,
 	/* Note, these may recurse into report_load_underway */
         if (slot->content_view != NULL && tell_current_content_view) {
 		view = slot->content_view;
-		uri = g_file_get_uri (location);
-		nautilus_view_load_location (slot->content_view, uri);
-		g_free (uri);
+		nautilus_view_load_location (slot->content_view, location);
         }
 	
         if (slot->new_content_view != NULL && tell_new_content_view &&
 	    (!tell_current_content_view ||
 	     slot->new_content_view != slot->content_view) ) {
 		view = slot->new_content_view;
-		uri = g_file_get_uri (location);
-		nautilus_view_load_location (slot->new_content_view, uri);
-		g_free (uri);
+		nautilus_view_load_location (slot->new_content_view, location);
         }
 	if (view != NULL) {
 		/* slot->new_content_view might have changed here if
@@ -1446,9 +1341,6 @@ location_has_really_changed (NautilusWindowSlot *slot)
 		widget = nautilus_view_get_widget (slot->new_content_view);
 		/* Switch to the new content view. */
 		if (gtk_widget_get_parent (widget) == NULL) {
-			if (slot->content_view != NULL) {
-				nautilus_window_slot_disconnect_content_view (slot, slot->content_view);
-			}
 			nautilus_window_slot_set_content_view_widget (slot, slot->new_content_view);
 		}
 		g_object_unref (slot->new_content_view);
@@ -1533,12 +1425,13 @@ nautilus_window_slot_show_x_content_bar (NautilusWindowSlot *slot, GMount *mount
 }
 
 static void
-nautilus_window_slot_show_trash_bar (NautilusWindowSlot *slot,
-				     NautilusWindow *window)
+nautilus_window_slot_show_trash_bar (NautilusWindowSlot *slot)
 {
 	GtkWidget *bar;
+	NautilusView *view;
 
-	bar = nautilus_trash_bar_new (window);
+	view = nautilus_window_slot_get_current_view (slot);
+	bar = nautilus_trash_bar_new (view);
 	gtk_widget_show (bar);
 
 	nautilus_window_slot_add_extra_location_widget (slot, bar);
@@ -1582,14 +1475,11 @@ found_mount_cb (GObject *source_object,
 {
 	FindMountData *data = user_data;
 	GMount *mount;
-	NautilusWindowSlot *slot;
 
 	if (g_cancellable_is_cancelled (data->cancellable)) {
 		goto out;
 	}
 
-	slot = data->slot;
-	
 	mount = g_file_find_enclosing_mount_finish (G_FILE (source_object),
 						    res,
 						    NULL);
@@ -1673,7 +1563,7 @@ update_for_new_location (NautilusWindowSlot *slot)
 		nautilus_window_slot_update_query_editor (slot);
 
 		if (nautilus_directory_is_in_trash (directory)) {
-			nautilus_window_slot_show_trash_bar (slot, window);
+			nautilus_window_slot_show_trash_bar (slot);
 		}
 
 		/* need the mount to determine if we should put up the x-content cluebar */
@@ -1747,15 +1637,11 @@ nautilus_window_report_load_complete (NautilusWindow *window,
 static void
 end_location_change (NautilusWindowSlot *slot)
 {
-	NautilusWindow *window;
 	char *uri;
 
-	window = slot->pane->window;
-
 	uri = nautilus_window_slot_get_location_uri (slot);
 	if (uri) {
-		nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
-				    "finished loading window %p: %s", window, uri);
+		DEBUG ("Finished loading window for uri %s", uri);
 		g_free (uri);
 	}
 
@@ -1807,7 +1693,7 @@ free_location_change (NautilusWindowSlot *slot)
 		nautilus_view_stop_loading (slot->new_content_view);
 		window->details->temporarily_ignore_view_signals = FALSE;
 
-		nautilus_window_slot_disconnect_content_view (slot, slot->new_content_view);
+		nautilus_window_disconnect_content_view (window, slot->new_content_view);
         	g_object_unref (slot->new_content_view);
                 slot->new_content_view = NULL;
         }
@@ -1860,7 +1746,6 @@ nautilus_window_report_view_failed (NautilusWindow *window,
 	fallback_load_location = NULL;
 	
 	if (view == slot->content_view) {
-		nautilus_window_slot_disconnect_content_view (slot, view);
                 nautilus_window_slot_set_content_view_widget (slot, NULL);
 
                 report_current_content_view_failure_to_user (slot);
@@ -1900,7 +1785,6 @@ display_view_selection_failure (NautilusWindow *window, NautilusFile *file,
 	char *error_message;
 	char *detail_message;
 	char *scheme_string;
-	GtkDialog *dialog;
 
 	/* Some sort of failure occurred. How 'bout we tell the user? */
 	full_uri_for_display = g_file_get_parse_name (location);
@@ -1988,8 +1872,8 @@ display_view_selection_failure (NautilusWindow *window, NautilusFile *file,
 		detail_message = g_strdup_printf (_("Error: %s\nPlease select another viewer and try again."), error->message);
 	}
 	
-	dialog = eel_show_error_dialog (error_message, detail_message, NULL);
-	
+	eel_show_error_dialog (error_message, detail_message, NULL);
+
 	g_free (uri_for_display);
 	g_free (error_message);
 	g_free (detail_message);
@@ -2031,9 +1915,7 @@ nautilus_window_slot_set_content_view (NautilusWindowSlot *slot,
 	g_assert (NAUTILUS_IS_WINDOW (window));
   
 	uri = nautilus_window_slot_get_location_uri (slot);
-	nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
-			    "change view of window %p: \"%s\" to \"%s\"",
-			    window, uri, id);
+	DEBUG ("Change view of window %s to %s", uri, id);
 	g_free (uri);
 
 	if (nautilus_window_slot_content_view_matches_iid (slot, id)) {
@@ -2064,7 +1946,7 @@ nautilus_window_manage_views_close_slot (NautilusWindowPane *pane,
 					 NautilusWindowSlot *slot)
 {
 	if (slot->content_view != NULL) {
-		nautilus_window_slot_disconnect_content_view (slot, slot->content_view);
+		nautilus_window_disconnect_content_view (slot->pane->window, slot->content_view);
 	}
 
 	free_location_change (slot);
@@ -2155,12 +2037,3 @@ nautilus_window_slot_reload (NautilusWindowSlot *slot)
 	g_object_unref (location);
 	g_list_free_full (selection, g_object_unref);
 }
-
-void
-nautilus_window_reload (NautilusWindow *window)
-{
-	g_assert (NAUTILUS_IS_WINDOW (window));
-
-	nautilus_window_slot_reload (window->details->active_pane->active_slot);
-}
-
