@@ -49,6 +49,7 @@
 #include <libnautilus-private/nautilus-ui-utilities.h>
 #include <libnautilus-private/nautilus-module.h>
 #include <libnautilus-private/nautilus-undo-manager.h>
+#include <libnautilus-private/nautilus-program-choosing.h>
 #include <libnautilus-private/nautilus-search-directory.h>
 #include <libnautilus-private/nautilus-search-engine.h>
 #include <libnautilus-private/nautilus-signaller.h>
@@ -66,7 +67,7 @@
 typedef struct {
         NautilusBookmark *bookmark;
         NautilusWindow *window;
-        guint changed_handler_id;
+	GCallback refresh_callback;
 	NautilusBookmarkFailedCallback failed_callback;
 } BookmarkHolder;
 
@@ -82,14 +83,17 @@ bookmark_holder_new (NautilusBookmark *bookmark,
 	new_bookmark_holder->window = window;
 	new_bookmark_holder->bookmark = bookmark;
 	new_bookmark_holder->failed_callback = failed_callback;
+	new_bookmark_holder->refresh_callback = refresh_callback;
 	/* Ref the bookmark because it might be unreffed away while 
 	 * we're holding onto it (not an issue for window).
 	 */
 	g_object_ref (bookmark);
-	new_bookmark_holder->changed_handler_id = 
-		g_signal_connect_object (bookmark, "appearance_changed",
-					 refresh_callback,
-					 window, G_CONNECT_SWAPPED);
+	g_signal_connect_object (bookmark, "notify::icon",
+				 refresh_callback,
+				 window, G_CONNECT_SWAPPED);
+	g_signal_connect_object (bookmark, "notify::name",
+				 refresh_callback,
+				 window, G_CONNECT_SWAPPED);
 
 	return new_bookmark_holder;
 }
@@ -97,8 +101,8 @@ bookmark_holder_new (NautilusBookmark *bookmark,
 static void
 bookmark_holder_free (BookmarkHolder *bookmark_holder)
 {
-	g_signal_handler_disconnect (bookmark_holder->bookmark,
-				     bookmark_holder->changed_handler_id);
+	g_signal_handlers_disconnect_by_func (bookmark_holder->bookmark,
+					      bookmark_holder->refresh_callback, bookmark_holder->window);
 	g_object_unref (bookmark_holder->bookmark);
 	g_free (bookmark_holder);
 }
@@ -164,9 +168,9 @@ nautilus_menus_append_bookmark_to_menu (NautilusWindow *window,
 {
 	BookmarkHolder *bookmark_holder;
 	char action_name[128];
-	char *name;
+	const char *name;
 	char *path;
-	GdkPixbuf *pixbuf;
+	GIcon *icon;
 	GtkAction *action;
 	GtkWidget *menuitem;
 
@@ -177,7 +181,7 @@ nautilus_menus_append_bookmark_to_menu (NautilusWindow *window,
 	name = nautilus_bookmark_get_name (bookmark);
 
 	/* Create menu item with pixbuf */
-	pixbuf = nautilus_bookmark_get_pixbuf (bookmark, GTK_ICON_SIZE_MENU);
+	icon = nautilus_bookmark_get_icon (bookmark);
 
 	g_snprintf (action_name, sizeof (action_name), "%s%d", parent_id, index_in_parent);
 
@@ -187,7 +191,7 @@ nautilus_menus_append_bookmark_to_menu (NautilusWindow *window,
 				 NULL);
 	
 	g_object_set_data_full (G_OBJECT (action), "menu-icon",
-				g_object_ref (pixbuf),
+				icon,
 				g_object_unref);
 
 	g_signal_connect_data (action, "activate",
@@ -214,9 +218,7 @@ nautilus_menus_append_bookmark_to_menu (NautilusWindow *window,
 	gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (menuitem),
 						   TRUE);
 
-	g_object_unref (pixbuf);
 	g_free (path);
-	g_free (name);
 }
 
 static void
@@ -261,8 +263,12 @@ static void
 action_undo_callback (GtkAction *action, 
 		      gpointer user_data) 
 {
-	nautilus_undo_manager_undo
-		(NAUTILUS_WINDOW (user_data)->application->undo_manager);
+	NautilusApplication *app;
+
+	app = nautilus_application_dup_singleton ();
+	nautilus_undo_manager_undo (app->undo_manager);
+
+	g_object_unref (app);
 }
 
 static void
@@ -358,7 +364,10 @@ static void
 action_reload_callback (GtkAction *action, 
 			gpointer user_data) 
 {
-	nautilus_window_reload (NAUTILUS_WINDOW (user_data));
+	NautilusWindowSlot *slot;
+
+	slot = nautilus_window_get_active_slot (NAUTILUS_WINDOW (user_data));
+	nautilus_window_slot_reload (slot);
 }
 
 static void
@@ -397,7 +406,7 @@ action_show_hidden_files_callback (GtkAction *action,
 		mode = NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_DISABLE;
 	}
 
-	nautilus_window_info_set_hidden_files_mode (window, mode);
+	nautilus_window_set_hidden_files_mode (window, mode);
 }
 
 static void
@@ -419,7 +428,7 @@ show_hidden_files_preference_callback (gpointer callback_data)
 		g_signal_handlers_unblock_by_func (action, action_show_hidden_files_callback, window);
 
 		/* inform views */
-		nautilus_window_info_set_hidden_files_mode (window, NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_DEFAULT);
+		nautilus_window_set_hidden_files_mode (window, NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_DEFAULT);
 
 	}
 }
@@ -531,7 +540,11 @@ static void
 action_up_callback (GtkAction *action, 
 		     gpointer user_data) 
 {
-	nautilus_window_go_up (NAUTILUS_WINDOW (user_data), FALSE, should_open_in_new_tab ());
+	NautilusWindow *window = user_data;
+	NautilusWindowSlot *slot;
+
+	slot = nautilus_window_get_active_slot (window);
+	nautilus_window_slot_go_up (slot, FALSE, should_open_in_new_tab ());
 }
 
 static void
@@ -546,9 +559,7 @@ action_nautilus_manual_callback (GtkAction *action,
 	window = NAUTILUS_WINDOW (user_data);
 
 	if (NAUTILUS_IS_DESKTOP_WINDOW (window)) {
-		gdk_spawn_command_line_on_screen (
-			gtk_window_get_screen (GTK_WINDOW (window)),
-			"gnome-help", &error);
+		nautilus_launch_application_from_command (gtk_window_get_screen (GTK_WINDOW (window)), "gnome-help", FALSE, NULL);
 	} else {
 		gtk_show_uri (gtk_window_get_screen (GTK_WINDOW (window)),
 			      "ghelp:user-guide#gosnautilus-1",
@@ -683,7 +694,7 @@ connect_proxy_cb (GtkUIManager *manager,
 		  GtkWidget *proxy,
 		  NautilusWindow *window)
 {
-	GdkPixbuf *icon;
+	GIcon *icon;
 	GtkWidget *widget;
 	
 	if (GTK_IS_MENU_ITEM (proxy)) {
@@ -697,19 +708,19 @@ connect_proxy_cb (GtkUIManager *manager,
 		icon = g_object_get_data (G_OBJECT (action), "menu-icon");
 		if (icon != NULL) {
 			gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (proxy),
-						       gtk_image_new_from_pixbuf (icon));
+						       gtk_image_new_from_gicon (icon, GTK_ICON_SIZE_MENU));
 		}
 	}
 	if (GTK_IS_TOOL_BUTTON (proxy)) {
 		icon = g_object_get_data (G_OBJECT (action), "toolbar-icon");
 		if (icon != NULL) {
-			widget = gtk_image_new_from_pixbuf (icon);
+			widget = gtk_image_new_from_gicon (icon, GTK_ICON_SIZE_LARGE_TOOLBAR);
 			gtk_widget_show (widget);
 			gtk_tool_button_set_icon_widget (GTK_TOOL_BUTTON (proxy),
 							 widget);
 		}
 	}
-	
+
 	widget = get_event_widget (proxy);
 	if (widget) {
 		g_signal_connect (widget, "button-press-event",
@@ -821,7 +832,7 @@ static const GtkActionEntry main_entries[] = {
   /* tooltip */                  N_("Connect to a remote computer or shared disk"),
                                  G_CALLBACK (action_connect_to_server_callback) },
   /* name, stock id */         { "Home", NAUTILUS_ICON_HOME,
-  /* label, accelerator */       N_("_Home Folder"), "<alt>Home",
+  /* label, accelerator */       N_("_Home"), "<alt>Home",
   /* tooltip */                  N_("Open your personal folder"),
                                  G_CALLBACK (action_home_callback) },
   /* name, stock id */         { "Go to Computer", NAUTILUS_ICON_COMPUTER,
