@@ -32,9 +32,10 @@
 #include <gtk/gtk.h>
 #include <eel/eel-glib-extensions.h>
 #include <libnautilus-private/nautilus-bookmark.h>
-#include <libnautilus-private/nautilus-window-info.h>
 #include <libnautilus-private/nautilus-search-directory.h>
-#include "nautilus-application.h"
+
+#include "nautilus-view.h"
+#include "nautilus-window-types.h"
 
 #define NAUTILUS_TYPE_WINDOW nautilus_window_get_type()
 #define NAUTILUS_WINDOW(obj) \
@@ -48,33 +49,32 @@
 #define NAUTILUS_WINDOW_GET_CLASS(obj) \
   (G_TYPE_INSTANCE_GET_CLASS ((obj), NAUTILUS_TYPE_WINDOW, NautilusWindowClass))
 
-#ifndef NAUTILUS_WINDOW_DEFINED
-#define NAUTILUS_WINDOW_DEFINED
-typedef struct NautilusWindow NautilusWindow;
-#endif
+typedef enum {
+        NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_DEFAULT,
+        NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_ENABLE,
+        NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_DISABLE
+} NautilusWindowShowHiddenFilesMode;
 
-#ifndef NAUTILUS_WINDOW_SLOT_DEFINED
-#define NAUTILUS_WINDOW_SLOT_DEFINED
-typedef struct NautilusWindowSlot NautilusWindowSlot;
-#endif
-
-typedef struct _NautilusWindowPane      NautilusWindowPane;
-
-typedef struct NautilusWindowSlotClass NautilusWindowSlotClass;
-typedef enum NautilusWindowOpenSlotFlags NautilusWindowOpenSlotFlags;
+typedef enum {
+        NAUTILUS_WINDOW_SPATIAL,
+        NAUTILUS_WINDOW_NAVIGATION,
+        NAUTILUS_WINDOW_DESKTOP
+} NautilusWindowType;
 
 GType          nautilus_window_slot_get_type (void);
 
+#include "nautilus-application.h"
+
 typedef enum {
         NAUTILUS_WINDOW_NOT_SHOWN,
         NAUTILUS_WINDOW_POSITION_SET,
         NAUTILUS_WINDOW_SHOULD_SHOW
 } NautilusWindowShowState;
 
-enum NautilusWindowOpenSlotFlags {
+typedef enum {
 	NAUTILUS_WINDOW_OPEN_SLOT_NONE = 0,
 	NAUTILUS_WINDOW_OPEN_SLOT_APPEND = 1
-};
+}  NautilusWindowOpenSlotFlags;
 
 typedef struct NautilusWindowDetails NautilusWindowDetails;
 
@@ -86,7 +86,6 @@ typedef struct {
 
 	/* Function pointers for overriding, without corresponding signals */
 
-        char * (* get_title) (NautilusWindow *window);
         void   (* sync_title) (NautilusWindow *window,
 			       NautilusWindowSlot *slot);
         NautilusIconInfo * (* get_icon) (NautilusWindow *window,
@@ -95,7 +94,6 @@ typedef struct {
         void   (* sync_allow_stop) (NautilusWindow *window,
 				    NautilusWindowSlot *slot);
 	void   (* set_allow_up) (NautilusWindow *window, gboolean allow);
-	void   (* reload)              (NautilusWindow *window);
         void   (* prompt_for_location) (NautilusWindow *window, const char *initial);
         void   (* get_min_size) (NautilusWindow *window, guint *default_width, guint *default_height);
         void   (* get_default_size) (NautilusWindow *window, guint *default_width, guint *default_height);
@@ -109,15 +107,15 @@ typedef struct {
 						  NautilusWindowSlot *slot);
 
         /* Signals used only for keybindings */
-        gboolean (* go_up) (NautilusWindow *window, gboolean close);
+        gboolean (* go_up)  (NautilusWindow *window,
+                             gboolean close);
+	void     (* reload) (NautilusWindow *window);
 } NautilusWindowClass;
 
 struct NautilusWindow {
         GtkWindow parent_object;
         
         NautilusWindowDetails *details;
-        
-        NautilusApplication *application;
 };
 
 GType            nautilus_window_get_type             (void);
@@ -135,26 +133,34 @@ void             nautilus_window_go_to_full           (NautilusWindow    *window
                                                        GFile             *location,
                                                        NautilusWindowGoToCallback callback,
                                                        gpointer           user_data);
-void             nautilus_window_go_to_with_selection (NautilusWindow    *window,
-                                                       GFile             *location,
-                                                       GList             *new_selection);
-void             nautilus_window_go_home              (NautilusWindow    *window);
 void             nautilus_window_new_tab              (NautilusWindow    *window);
-void             nautilus_window_go_up                (NautilusWindow    *window,
-                                                       gboolean           close_behind,
-						       gboolean           new_tab);
 void             nautilus_window_prompt_for_location  (NautilusWindow    *window,
                                                        const char        *initial);
 void             nautilus_window_display_error        (NautilusWindow    *window,
                                                        const char        *error_msg);
-void		 nautilus_window_reload		      (NautilusWindow	 *window);
 
-void             nautilus_window_allow_reload         (NautilusWindow    *window,
-                                                       gboolean           allow);
 void             nautilus_window_allow_up             (NautilusWindow    *window, 
                                                        gboolean           allow);
-void             nautilus_window_allow_stop           (NautilusWindow    *window, 
-                                                       gboolean           allow);
 GtkUIManager *   nautilus_window_get_ui_manager       (NautilusWindow    *window);
 
+
+void                 nautilus_window_report_load_complete     (NautilusWindow *window,
+                                                               NautilusView *view);
+
+NautilusWindowSlot * nautilus_window_get_extra_slot       (NautilusWindow *window);
+NautilusWindowShowHiddenFilesMode
+                     nautilus_window_get_hidden_files_mode (NautilusWindow *window);
+void                 nautilus_window_set_hidden_files_mode (NautilusWindow *window,
+                                                            NautilusWindowShowHiddenFilesMode  mode);
+NautilusWindowType   nautilus_window_get_window_type       (NautilusWindow *window);
+void                 nautilus_window_report_load_underway  (NautilusWindow *window,
+                                                            NautilusView *view);
+void                 nautilus_window_view_visible          (NautilusWindow *window,
+                                                            NautilusView *view);
+void                 nautilus_window_set_initiated_unmount (NautilusWindow *window,
+                                                            gboolean initiated_unmount);
+NautilusWindowSlot * nautilus_window_get_active_slot       (NautilusWindow *window);
+void                 nautilus_window_push_status           (NautilusWindow *window,
+                                                            const char *text);
+
 #endif
